/************************************************************************************
The code is designed and produced by MDY Science and Education Co., Ltd, which has the entire ownership. It is only for personal learning, which cannot be used for commercial or profit-making purposes without permission.

    MDY's Mission: Develop Chip Talents and Realize National Chip Dream.

    We sincerely hope that our students can learn the real IC / FPGA code through our standard and rigorous code.

    For more FPGA learning materials, please visit the Forum: http://fpgabbs.com/ and official website: http://www.mdy-edu.com/index.html 

    *************************************************************************************/

module key(
    clk             ,  
    rst_n           ,   

    slide_switch    ,  
    push_button     ,  

    key_num         ,   
    key_vld             
    
    );

    input               clk                 ;
    input               rst_n               ;
    input [9:0]         slide_switch        ;
    input               push_button         ;

    output[3:0]         key_num             ;
    output              key_vld             ;

    reg   [3:0]         key_num             ;
    reg                 key_vld             ;



    
    //slide_switch_dff
    reg [9:0] slide_switch_dff0,slide_switch_dff1,slide_switch_dff2;
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            slide_switch_dff0 <= 0;
            slide_switch_dff1 <= 0;
            slide_switch_dff2 <= 0;
        end
        else begin
            slide_switch_dff0 <= slide_switch;
            slide_switch_dff1 <= slide_switch_dff0;
            slide_switch_dff2 <= slide_switch_dff1;
        end
    end

    //cnt_20ms
    parameter TIME_20MS = 1_000_000         ;
    reg   [19:0]        cnt_20ms            ;
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            cnt_20ms <= 0;
        end
        else if(clr_cnt_20ms)begin
            cnt_20ms <= 0;
        end
        else if(add_cnt_20ms)begin
            cnt_20ms <= cnt_20ms + 1;
        end
    end
    assign add_cnt_20ms  = cnt_20ms<TIME_20MS-1;
    assign clr_cnt_20ms  = slide_switch_dff2!=slide_switch_dff1;    
    assign full_cnt_20ms = cnt_20ms==TIME_20MS-1;

    //slide_switch_high
    reg   [9:0]         slide_switch_high   ;
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            slide_switch_high <= 0;
        end
        else if(full_cnt_20ms)begin
            slide_switch_high <= slide_switch_dff2;
        end
    end

    //slide_switch_high_dff0
    reg   [9:0]         slide_switch_high_dff0;
    wire  [9:0]         slide_switch_high_vld;
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            slide_switch_high_dff0 <= 0;
        end
        else begin
            slide_switch_high_dff0 <= slide_switch_high;
        end
    end
    assign slide_switch_high_vld = (~slide_switch_high_dff0) & slide_switch_high;



    //key_num
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            key_num <= 0;
        end
        else if(slide_switch_high_vld[0])begin
            key_num <= 0;
        end
        else if(slide_switch_high_vld[1])begin
            key_num <= 1;
        end
        else if(slide_switch_high_vld[2])begin
            key_num <= 2;
        end
        else if(slide_switch_high_vld[3])begin
            key_num <= 3;
        end
        else if(slide_switch_high_vld[4])begin
            key_num <= 4;
        end
        else if(slide_switch_high_vld[5])begin
            key_num <= 5;
        end
        else if(slide_switch_high_vld[6])begin
            key_num <= 6;
        end
        else if(slide_switch_high_vld[7])begin
            key_num <= 7;
        end
        else if(slide_switch_high_vld[8])begin
            key_num <= 8;
        end
        else if(slide_switch_high_vld[9])begin
            key_num <= 9;
        end
        else if(push_button_vld)begin
            key_num <= 10;
        end
    end

    //key_vld
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            key_vld <= 0;
        end
        else begin
            key_vld <= (|slide_switch_high_vld) || push_button_vld;
        end
    end


    endmodule

